==> audiochain-1.1.5-src.tgz: org/audiochain/io/MixingAudioDataReader.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

import static org.audiochain.io.AudioDataUtil.*;

import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;

public class MixingAudioDataReader implements AudioDataReader, Dumpable {

	private Collection<AudioDataReader> readers;
	private AudioDataReader singleReader;
	private int[] buffer;
	private int bufferSize;
	private boolean seekRequested;
	private long requestedSeekPosition;

	public MixingAudioDataReader(AudioDataReader... readers) {
		if (readers.length == 0) {
			throw new IllegalArgumentException("There must be at least one reader.");
		} else if (readers.length == 1) {
			singleReader = readers[0];
		} else {
			this.readers = new LinkedList<AudioDataReader>(Arrays.asList(readers));
		}
	}

	public MixingAudioDataReader(Collection<AudioDataReader> readers) {
		if (readers.isEmpty()) {
			throw new IllegalArgumentException("There must be at least one reader.");
		} else if (readers.size() == 1) {
			singleReader = readers.iterator().next();
		} else {
			this.readers = new LinkedList<AudioDataReader>(readers);
		}
	}

	@Override
	public int read(int[] data) throws IOException {
		if (singleReader != null) {
			if(seekRequested) {
				seekRequested = false;
				singleReader.seek(requestedSeekPosition);
			}
			return singleReader.read(data);
		}
		
		if (buffer == null) {
			bufferSize = data.length;
			buffer = new int[bufferSize];
		} else {
			Arrays.fill(buffer, 0);
		}
		// get a clean basis
		Arrays.fill(data, 0);
		
		if(seekRequested) {
			seekRequested = false;
			for (AudioDataReader reader : readers) {
				reader.seek(requestedSeekPosition);
			}
		}
		
		Iterator<AudioDataReader> iter = readers.iterator();
		AudioDataReader reader = iter.next();
		int maxReadCount = readFully(reader, data);
		while (iter.hasNext()) {
			reader = iter.next();
			int readCount = readFully(reader, buffer);
			
			maxReadCount = Math.max(maxReadCount, readCount);
			for (int i = 0; i < readCount; i++) {
				int a = buffer[i];
				int b = data[i];
				data[i] = a + b;
			}
		}
		
		return maxReadCount;
	}

	@Override
	public void close() throws IOException {
		if (readers != null) {
			for (AudioDataReader reader : readers) {
				reader.close();
			}
			readers = null;
		}
		if (singleReader != null) {
			singleReader.close();
			singleReader = null;
		}
		buffer = null;
	}

	@Override
	public void stop() {
		if (singleReader != null) {
			singleReader.stop();
		}
		if (readers != null) {
			for (AudioDataReader reader : new HashSet<AudioDataReader>(readers)) {
				reader.stop();
			}
		}
	}

	@Override
	public void seek(long framePosition) throws IOException {
		seekRequested = true;
		requestedSeekPosition = framePosition;
	}
	
	@Override
	public void dump(Appendable a) throws IOException {
		a.append(String.valueOf(this)).append('\n');
		if (readers != null) {
			int branchCount = 0;
			for (AudioDataReader reader : readers) {
				a.append("Mix branch " + ++branchCount + ":\n");
				AudioDataUtil.dumpAudioDataReaderChain(reader, a);
			}
		}
		if (singleReader != null) {
			AudioDataUtil.dumpAudioDataReaderChain(singleReader, a);
		}
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/io/MixingAudioDataReader.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

import static org.audiochain.io.AudioDataUtil.readFully;

import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;

public class MixingAudioDataReader implements AudioDataReader, Dumpable {

	private Collection<AudioDataReader> readers;
	private AudioDataReader singleReader;
	private int[] buffer;
	private int bufferSize;
	private boolean seekRequested;
	private long requestedSeekPosition;

	public MixingAudioDataReader(AudioDataReader... readers) {
		if (readers.length == 0) {
			throw new IllegalArgumentException("There must be at least one reader.");
		} else if (readers.length == 1) {
			singleReader = readers[0];
		} else {
			this.readers = new LinkedList<AudioDataReader>(Arrays.asList(readers));
		}
	}

	public MixingAudioDataReader(Collection<AudioDataReader> readers) {
		if (readers.isEmpty()) {
			throw new IllegalArgumentException("There must be at least one reader.");
		} else if (readers.size() == 1) {
			singleReader = readers.iterator().next();
		} else {
			this.readers = new LinkedList<AudioDataReader>(readers);
		}
	}

	@Override
	public int read(int[] data) throws IOException {
		if (singleReader != null) {
			if(seekRequested) {
				seekRequested = false;
				singleReader.seek(requestedSeekPosition);
			}
			return singleReader.read(data);
		}
		
		if (buffer == null) {
			bufferSize = data.length;
			buffer = new int[bufferSize];
		} else {
			Arrays.fill(buffer, 0);
		}
		// get a clean basis
		Arrays.fill(data, 0);
		
		if(seekRequested) {
			seekRequested = false;
			for (AudioDataReader reader : readers) {
				reader.seek(requestedSeekPosition);
			}
		}
		
		Iterator<AudioDataReader> iter = readers.iterator();
		AudioDataReader reader = iter.next();
		int maxReadCount = readFully(reader, data);
		while (iter.hasNext()) {
			reader = iter.next();
			int readCount = readFully(reader, buffer);
			
			maxReadCount = Math.max(maxReadCount, readCount);
			for (int i = 0; i < readCount; i++) {
				int a = buffer[i];
				int b = data[i];
				data[i] = a + b;
			}
		}
		
		return maxReadCount;
	}

	@Override
	public void close() throws IOException {
		if (readers != null) {
			for (AudioDataReader reader : readers) {
				reader.close();
			}
			readers = null;
		}
		if (singleReader != null) {
			singleReader.close();
			singleReader = null;
		}
		buffer = null;
	}

	@Override
	public void stop() {
		if (singleReader != null) {
			singleReader.stop();
		}
		if (readers != null) {
			for (AudioDataReader reader : new HashSet<AudioDataReader>(readers)) {
				reader.stop();
			}
		}
	}

	@Override
	public void seek(long framePosition) throws IOException {
		seekRequested = true;
		requestedSeekPosition = framePosition;
	}
	
	@Override
	public void dump(Appendable a) throws IOException {
		a.append(String.valueOf(this)).append('\n');
		if (readers != null) {
			int branchCount = 0;
			for (AudioDataReader reader : readers) {
				a.append("Mix branch " + ++branchCount + ":\n");
				AudioDataUtil.dumpAudioDataReaderChain(reader, a);
			}
		}
		if (singleReader != null) {
			AudioDataUtil.dumpAudioDataReaderChain(singleReader, a);
		}
	}
}
